require './test/test_helper'
require 'yaml'

describe Divy::Grid do

  before do
    @grid = Divy::Grid.new
    @html_results = YAML.load(File.open('test/lib/divy/html_examples.yml'))
  end

  describe '#initialize' do
    it 'should set content to the given value if valid' do
      content = [{type: :html, value: 'totally valid', location: [1, 3]}]
      new_grid = Divy::Grid.new(content: content)
      new_grid.content.must_equal content
    end

    it 'should set the default value for content if none given' do
      @grid.content.must_equal([{:type => :none, :value => '', :location => [0, 0], :size=>[1, 1]}])
    end

    it 'should set container size if valid' do
      new_grid = Divy::Grid.new(container_size: [20, 20])
      new_grid.container_size.must_equal [20, 20]
    end

    it 'should set default container size if none given' do
      @grid.container_size.must_equal %w(100% 100%)
    end

    it 'should set grid dimensions if valid' do
      new_grid = Divy::Grid.new(grid_dimensions: [7, 7])
      new_grid.grid_dimensions.must_equal [7, 7]
    end

    it 'should set default grid dimensions if none given' do
      @grid.grid_dimensions.must_equal [10,10]
    end

    it 'should set grid color if valid' do
      new_grid = Divy::Grid.new(grid_color: '#000000')
      new_grid.grid_color.must_equal '#000000'
    end

    it 'should set default grid color to invisible if none given' do
      @grid.grid_color.must_equal 'invisible'
    end

    it 'should set show_grid to false if not provided' do
      @grid.show_grid.must_equal false
    end
  end

  describe '#add_content' do
    it 'should add content to the content array if valid' do
      content = [{type: :html, value: 'coolhtmlstuff', location: [1, 3], size: [1, 1]}]
      result_content = [
        {type: :none, value: '', location: [0, 0], size: [1, 1]},
        content[0]
      ]
      @grid.add_content(content)
      @grid.content.must_equal result_content
    end
  end

  describe '#validated_content_arr' do
    it 'should return all content if all valid' do
      content = [
        {type: :html, value: 'coolhtmlstuff', location: [1, 3]},
        {type: :image, value: 'someimagepath', location: [1, 3]},
        {type: :none, value: 'gonnaignorethisanyways', location: [1, 3]}
      ]
      @grid.validated_content_arr(content).must_equal content
    end

    it 'should raise an error if invalid content is provided' do
      content = [
        {type: :html, value: 'coolhtmlstuff', location: [1, 3]},
        {type: :image, value: 'someimagepath', location: [1, 3]},
        {broken_stuff: 'HAHAIBREAKINURCODEZ!'}
      ]
      proc{@grid.validated_content_arr(content)}.must_raise DivyGridError
    end
  end

  describe '#validated_content_hsh' do
    it 'should return content if content is valid html type' do
      content = {type: :html, value: 'coolhtmlstuff', location: [1, 3]}
      @grid.validated_content_hsh(content).must_equal content
    end

    it 'should return content if content is valid html type' do
      content = {type: :image, value: 'someimagepath', location: [1, 3]}
      @grid.validated_content_hsh(content).must_equal content
    end

    it 'should return content if content is valid html type' do
      content = {type: :none, value: 'gonnaignorethisanyways', location: [1, 3]}
      @grid.validated_content_hsh(content).must_equal content
    end

    it 'should raise an exception if the content type is not valid' do
      proc{@grid.validated_content_hsh(broken_stuff: 'HAHAIBREAKINURCODEZ!')}.must_raise DivyGridError
    end
  end

  describe '#validated hex color' do
    it 'should return the color if a valid hex color' do
      @grid.validated_hex_color('#FF0022').must_equal '#FF0022'
    end

    it 'should raise an exception if input is not a valid hex color' do
      proc{@grid.validated_hex_color('#FF002')}.must_raise DivyGridError
    end
  end

  describe '#validated_container_size' do
    it 'should return the container size if both inputs are valid percents as a string' do
      @grid.validated_container_size(%w(20% 88%)).must_equal ['20%', '88%']
    end

    it 'should return the container size if both inputs are valid ints' do
      @grid.validated_container_size([56, 97]).must_equal [56, 97]
    end

    it 'should return the container size if both inputs are valid ints as a string' do
      @grid.validated_container_size(%w(56 97)).must_equal [56, 97]
    end

    it 'should raise an exception if inputs are not valid ints or percents' do
      proc{@grid.validated_container_size([56, 'snoo'])}.must_raise DivyGridError
    end
  end

  describe '#validated_number_array' do
    it 'should return the array if both inputs in array are numbers as float/int' do
      @grid.validated_number_array([4, 8]).must_equal [4, 8]
    end

    it 'should return the array if the inputs are valid numbers as strings' do
      @grid.validated_number_array(%w(4 8)).must_equal [4, 8]
    end

    it 'should return the array for a valid input of a combo of int/float/string' do
      @grid.validated_number_array([9, '2']).must_equal [9, 2]
    end

    it 'should raise an exception if both inputs are not valid numbers' do
      proc{@grid.validated_number_array(%w(meow 8))}.must_raise DivyGridError
    end
  end

  describe '#two_numbers?' do
    it 'should return true if both inputs are integers/floats' do
      @grid.two_numbers?([2, 6]).must_equal true
    end

    it 'should return true if both inputs are numbers a as a string' do
      @grid.two_numbers?(%w(2 6)).must_equal true
    end

    it 'should return true for a combo of ints/floats/numbers as a string' do
      @grid.two_numbers?([9, '4']).must_equal true
    end

    it 'should return false if either input is not a valid number' do
      @grid.two_numbers?(['puppy', 6]).must_equal false
    end

    it 'should return false if array length is not 2' do
      @grid.two_numbers?([2, 6, 4]).must_equal false
    end
  end

  describe '#two_percents?' do
    it 'should return true if both numbers are valid percents as a string' do
      @grid.two_percents?(%w(46% 23%)).must_equal true
    end

    it 'should return false if either number is not a percent as a string' do
      @grid.two_percents?(%w(bleh 23%)).must_equal false
    end

    it 'should return false if array length is not 2' do
      @grid.two_percents?(%w(46% 23% 12%)).must_equal false
    end
  end

  describe '#html' do
    it 'should return correct html content given all arguments' do
      @grid.content = [
        {type: :html, value: 'coolhtmlstuff', location: [1, 3], size: [1, 1]},
        {type: :image, value: 'someimagepath', location: [1, 3], size: [2, 2]}
      ]
      @grid.grid_dimensions = [5, 5]
      @grid.show_grid = true
      @grid.grid_color = '#FF00CC'
      @grid.container_size = [100, 100]
      @grid.html.must_equal (@html_results[0] + "\n")
    end

    it 'should return default contents given no arguments' do
      @grid.html.must_equal (@html_results[4] + "\n")
    end
  end

  describe '#content=' do
    it 'sets content if valid array' do
      @grid.content = [{type: :html, value: 'totally valid', location: [1, 3], size: [1, 1]}]
      @grid.content.must_equal([{type: :html, value: 'totally valid', location: [1, 3], size: [1, 1]}])
    end

    it 'sets content if valid hash' do
      @grid.content = {type: :html, value: 'totally valid', location: [1, 3], size: [1, 1]}
      @grid.content.must_equal([{type: :html, value: 'totally valid', location: [1, 3], size: [1, 1]}])
    end

    it 'does not change content if invalid' do
      @grid.content = 'not quite right'
      @grid.content.wont_equal 'not quite right'
    end
  end

  describe '#container_size=' do
    it 'sets container size if valid' do
      @grid.container_size = [4, 5]
      @grid.container_size.must_equal [4, 5]
    end

    it 'does not set container size if invalid' do
      @grid.container_size = [4, 5, 6, 7]
      @grid.container_size.wont_equal [4, 5, 6, 7]
    end
  end

  describe '#grid_dimensions=' do
    it 'sets grid dimensions if valid' do
      @grid.grid_dimensions = [4, 5]
      @grid.grid_dimensions.must_equal [4, 5]
    end

    it 'does not set grid dimensions if invalid' do
      @grid.grid_dimensions = [4, 5, 6, 7]
      @grid.grid_dimensions.wont_equal [4, 5, 6, 7]
    end
  end

  describe '#grid_color=' do
    it 'sets grid color if valid' do
      @grid.grid_color = '#FFFFFF'
      @grid.grid_color.must_equal '#FFFFFF'
    end

    it 'does not set grid color if invalid' do
      @grid.grid_color = 'notacolor'
      @grid.grid_color.wont_equal 'notacolor'
    end
  end

  describe '#show_grid=' do
    it 'sets show grid if a boolean' do
      @grid.show_grid = true
      @grid.show_grid.must_equal true
    end

    it 'does not set show grid if not a boolean' do
      @grid.show_grid = 'bunnies!'
      @grid.show_grid.wont_equal 'bunnie!'
    end
  end
end
